searchData={"items":[{"type":"module","title":"Nvir","doc":"This is the main API for Nvir, an environment variable loader and validator.\n\nThe most useful documentation is generally:\n\n* The [README](README.md#basic-usage) for usage instructions.\n* The the `dotenv!/1` function.","ref":"Nvir.html"},{"type":"type","title":"Nvir.config_opt/0","doc":"","ref":"Nvir.html#t:config_opt/0"},{"type":"function","title":"Nvir.default_dotenv_sources/0","doc":"Returns the sources enabed by default when using `dotenv/1` or\n`dotenv_loader/0`. The value changes dynamically depending on the current\nenvironment and operating system.\n\nSee the \"Predefined tags\" section on the `dotenv!/1` documentation.","ref":"Nvir.html#default_dotenv_sources/0"},{"type":"function","title":"Nvir.dotenv!/1","doc":"Loads specified dotenv files in the system environment. Intended usage is from\n`config/runtime.exs` in your project\n\nVariables defined in the files will not overwrite the system environment if\nthey are already defined. To overwrite the system env, please list your files\nunder an `:overwrite` key.\n\nThis function takes multiple sources and will select the sources to actually\nload based on system properties.\n\nValid sources are:\n\n* A string, this is an actual file that we want to load.\n* A `{tag, value}` tuple where the tag is an atom and the value is a source.\n  Predefined tags are listed below. Additional tags can be defined with\n  `enable_sources/3`.\n* A list of sources. So a keyword list is a valid source, _i.e._ a list of\n  tagged tuples.\n\nFiles are loaded in order of appearance, in two phases:\n\n* First, files that are not wrapped in an `:overwrite` tagged tuple.\n* Then files that are wrapped in such tuples.\n\nFiles that do not exist are safely ignored.","ref":"Nvir.html#dotenv!/1"},{"type":"function","title":"Examples - Nvir.dotenv!/1","doc":"import Config\n    import Nvir\n\n    # Load a single file\n    dotenv!(\".env\")\n\n    # Load multiple files\n    dotenv!([\".env\", \".env.#{config_env()}\"])\n\n    # Load files depending on environment\n    dotenv!(\n      dev: \".env.dev\",\n      test: \".env.test\"\n    )\n\n    # Load files with and without overwrite\n    dotenv!(\n      dev: \".env\",\n      test: [\".env\", \".env.test\"],\n      overwrite: [test: \".env.test.local\"]\n    )\n\n    # Overwrite the system with all existing files\n    dotenv!(\n      overwrite: [\n        dev: \".env\",\n        test: [\".env\", \".env.test\", \".env.test.local\"]\n      ]\n    )\n\n    # Totally useless but valid :)\n    dotenv!(test: [test: [test: \".env.test\"]])\n    # Same without wrapping the tuples in lists\n    dotenv!({:test, {:test, {:test, \".env.test\"}}})\n\n    # This will not load the file as `:test` and `:dev` will not be\n    # enabled at the same time\n    dotenv!(dev: [test: \".env.test\"])","ref":"Nvir.html#dotenv!/1-examples"},{"type":"function","title":"Predefined tags - Nvir.dotenv!/1","doc":"Tags are enabled under different circumstances.\n\n#### Mix environment\n\n* `:dev` - When `Config.config_env()` or `Mix.env()` is `:dev`.\n* `:test` - When `Config.config_env()` or `Mix.env()` is `:test`.\n\nThere is no predefined tag for `:prod` as using dotenv files in production is\nstrongly discouraged.\n\n#### Continuous integration\n\n* `:ci` - When the `CI` environment variable is `\"true\"`.\n* `:ci@github` - When the `GITHUB_ACTIONS` environment variable is `\"true\"`.\n* `:ci@travis` - When the `TRAVIS` environment variable is `\"true\"`.\n* `:ci@circle` - When the `CIRCLECI` environment variable is `\"true\"`.\n* `:ci@gitlab` - When the `GITLAB_CI` environment variable is `\"true\"`.\n\n#### Operating system\n\n* `:linux` - On Linux machines.\n* `:windows` - On Windows machines.\n* `:darwin` - On MacOS machines.","ref":"Nvir.html#dotenv!/1-predefined-tags"},{"type":"function","title":"Nvir.dotenv!/2","doc":"Same as `dotenv!/1` but accepts a custom configuration to load dotenv files.","ref":"Nvir.html#dotenv!/2"},{"type":"function","title":"Nvir.dotenv_configure/2","doc":"Updates the given configuration with the given options.\n\nThe options are not merged.","ref":"Nvir.html#dotenv_configure/2"},{"type":"function","title":"Options - Nvir.dotenv_configure/2","doc":"* `:enabled_sources` - A map of `%{atom => boolean}` values to declare which\n  source tags will be enabled when collecting sources. Defaults to the return\n  value of `default_enabled_sources/0`.\n* `:parser` - The module to parse environment variables files. Defaults to\n  `Nvir.Parser.RDB`.\n* `:cd` - A directory path to load relative source paths from.\n* `:before_env_set` - A function that accepts a `{varname, value}` tuple and\n  must return a similar tuple. This gives the possibility to change or\n  transform the parsed variables before the environment is altered. Returned\n  `varname` and `value` must implement the `String.Chars` protocol. Returning\n  `nil` as a value will delete the environment variable.","ref":"Nvir.html#dotenv_configure/2-options"},{"type":"function","title":"Example - Nvir.dotenv_configure/2","doc":"Implement loading a custom `:docs` environment and load a file when running a\nrelease:\n\n    import Nvir\n\n    dotenv_loader()\n    |> dotenv_configure(\n      enabled_sources: %{\n        # Enable sources tagged with :docs depending on an environment variable\n        docs: System.get_env(\"MIX_ENV\") == \"docs\",\n\n        # Enable sources tagged with :rel when running a release\n        rel: env!(\"RELEASE_NAME\", :boolean, false)\n      },\n\n      # Load dotenv files relative to this directory\n      cd: \"~/projects/apps/envs\"\n    )\n    |> dotenv!(\n      docs: \".env.docs\",\n      rel: \"releases.env\",\n      dev: \".env.dev\",\n      test: \".env.test\"\n    )","ref":"Nvir.html#dotenv_configure/2-example"},{"type":"function","title":"Nvir.dotenv_enable_sources/2","doc":"Like `dotenv_enable_sources/3` but accepts a keyword list or map of sources.\n\n    Nvir.dotenv_loader()\n    |> Nvir.dotenv_enable_source(\n      custom: true,\n      docs: config_env() == :docs\n    )\n    |> Nvir.dotenv!([\"global.env\", custom: \"local.env\", docs: \"docs.env\"])","ref":"Nvir.html#dotenv_enable_sources/2"},{"type":"function","title":"Nvir.dotenv_enable_sources/3","doc":"Enables or disables environment variable sources under the given tag.\n\nFor instance, the following call will load both files:\n\n    Nvir.dotenv_loader()\n    |> Nvir.enable_sources(:custom, true)\n    |> Nvir.dotenv!([\"global.env\", custom: \"local.env\"])\n\nWhereas the following call will only load files that are not wrapped in a tag.\n\n    Nvir.dotenv_loader()\n    |> Nvir.dotenv!([\"global.env\", custom: \"local.env\"])\n\nIt is also possible to disable some defaults by overriding them. In the\nfollowing code, the `.env.test` file will never be loaded:\n\n    Nvir.dotenv_loader()\n    |> Nvir.enable_sources(:test, false)\n    |> Nvir.dotenv!([\"global.env\", dev: \".env.dev\", test: \".env.test\"])","ref":"Nvir.html#dotenv_enable_sources/3"},{"type":"function","title":"Nvir.dotenv_loader/0","doc":"Returns the default configuration for the `dotenv!/2` function.","ref":"Nvir.html#dotenv_loader/0"},{"type":"function","title":"Examples - Nvir.dotenv_loader/0","doc":"Implement loading a custom `:docs` environment:\n\n    import Config\n    import Nvir\n\n    dotenv_loader()\n    |> enable_source(:docs, config_env() == :docs)\n    |> dotenv!(\n      docs: \".env.docs\",\n      dev: \".env.dev\",\n      test: \".env.test\"\n    )","ref":"Nvir.html#dotenv_loader/0-examples"},{"type":"function","title":"Nvir.dotenv_new/0","doc":"Returns a configuration for `dotenv!/2` without any enabled source.\n\nGenerally this is used for custom loading strategies, see `dotenv_loader/0` to\nuse reasonable defaults.","ref":"Nvir.html#dotenv_new/0"},{"type":"function","title":"Nvir.env!/2","doc":"Returns the value of the given `var`, transformed and validated by the given\n`caster`.\n\nRaises if the variable is not defined or if the caster validation fails.\n\nPlease see the [README](README.md#available-casters) for available casters.","ref":"Nvir.html#env!/2"},{"type":"function","title":"Nvir.env!/3","doc":"Returns the value of the given `var`, transformed and validated by the given\n`caster`.\n\nReturns the `default` value if the variable is not defined.\n\nRaises if the caster validation fails.\n\nPlease see the [README](README.md#available-casters) for available casters.","ref":"Nvir.html#env!/3"},{"type":"type","title":"Nvir.source/0","doc":"","ref":"Nvir.html#t:source/0"},{"type":"type","title":"Nvir.sources/0","doc":"","ref":"Nvir.html#t:sources/0"},{"type":"type","title":"Nvir.t/0","doc":"","ref":"Nvir.html#t:t/0"},{"type":"type","title":"Nvir.transformer/0","doc":"","ref":"Nvir.html#t:transformer/0"},{"type":"type","title":"Nvir.var_def/0","doc":"","ref":"Nvir.html#t:var_def/0"},{"type":"module","title":"Nvir.Cast","doc":"Collection of casters for environment variables.","ref":"Nvir.Cast.html"},{"type":"function","title":"Nvir.Cast.cast/2","doc":"Casts the given value to the desired type.\n\nEnvironment variables are always defined as a string. Thus, the `cast/2`\nfunction will only accept strings for the `value` argument.\n\nAccepts a built-in caster or a custom function returning `{:ok, value}` or\n`{:error, String.t()}`. You may as well directly return an error tuple from a\nrecursive `cast/2` call.","ref":"Nvir.Cast.html#cast/2"},{"type":"function","title":"Built-in casters - Nvir.Cast.cast/2","doc":"","ref":"Nvir.Cast.html#cast/2-built-in-casters"},{"type":"function","title":"String Casters - Nvir.Cast.cast/2","doc":"| Caster | Description |\n|--------|-------------|\n| `:string` | Returns the value as-is. |\n| `:string?` | Converts empty strings to `nil`. |\n| `:string!` | Raises for empty strings. |","ref":"Nvir.Cast.html#cast/2-string-casters"},{"type":"function","title":"Boolean Casters - Nvir.Cast.cast/2","doc":"| Caster | Description |\n|--------|-------------|\n| `:boolean` | `\"false\"`, `\"0\"` and empty strings become `false`, any other value is `true`. Case insensitive. It is recommended to use `:boolean!` instead. |\n| `:boolean!` | Accepts only `\"true\"`, `\"false\"`, `\"1\"`, and `\"0\"`. Case insensitive. |","ref":"Nvir.Cast.html#cast/2-boolean-casters"},{"type":"function","title":"Number Casters - Nvir.Cast.cast/2","doc":"| Caster | Description |\n|--------|-------------|\n| `:integer!` | Strict integer parsing. |\n| `:integer?` | Like `:integer!` but empty strings becomes `nil`. |\n| `:float!` | Strict float parsing. |\n| `:float?` | Like `:float!` but empty strings becomes `nil`. |","ref":"Nvir.Cast.html#cast/2-number-casters"},{"type":"function","title":"Atom Casters - Nvir.Cast.cast/2","doc":"| Caster | Description |\n|--------|-------------|\n| `:atom` | Converts the value to an atom. Use the `:existing_atom` variants when possible. |\n| `:atom?` | Like `:atom` but empty strings becomes `nil`. |\n| `:atom!` | Like `:atom` but rejects empty strings. |\n| `:existing_atom` | Converts to existing atom only, raises otherwise. |\n| `:existing_atom?` | Like `:existing_atom` but empty strings becomes `nil`. |\n| `:existing_atom!` | Like `:existing_atom` but rejects empty strings. |\n\nNote that using `:existing_atom` with empty strings will not raise an exception\nbecause the `:\"\"` atom is valid and is generally defined by the system on boot.","ref":"Nvir.Cast.html#cast/2-atom-casters"},{"type":"function","title":"Deprecated casters - Nvir.Cast.cast/2","doc":"Those exist for legacy reasons and should be avoided. They will trigger a\nruntime warning when used.\n\nIn some languages, using `null` where a number is expected will cast the value\nto a _default type value_, generally `0` and `+0.0` for integers and floats.\nThis behaviour does not exist in Elixir so casters for such types behave the\nsame with-or-without the `!` suffix. This means `:integer` and `:float` will\nraise for empty strings.\n\n| Caster | Description |\n|--------|-------------|\n| `:boolean?` | Same as `:boolean`. ⚠️ Returns `false` instead of `nil` for empty strings. |\n| `:integer` | Same as `:integer!`. |\n| `:float` | Same as `:float!`. |","ref":"Nvir.Cast.html#cast/2-deprecated-casters"},{"type":"type","title":"Nvir.Cast.caster/0","doc":"","ref":"Nvir.Cast.html#t:caster/0"},{"type":"type","title":"Nvir.Cast.result/0","doc":"","ref":"Nvir.Cast.html#t:result/0"},{"type":"behaviour","title":"Nvir.Parser","doc":"A behaviour for environment variables sources parser.\n\nThe default implementation used in Nvir is `Nvir.Parser.RDB`. It can only\nparse classic dotenv files.\n\nParsing other sources such as YAML files or encrypted files requires to\nprovide your own parser.\n\nSee the documentation of the `c:parse_file/1` callback for implementation\nguidelines.","ref":"Nvir.Parser.html"},{"type":"function","title":"Nvir.Parser.interpolate_var/2","doc":"Takes a parsed value returned by the parser implementation, and a resolver\nfor the interpolated variables.\n\nA resolver is a function that takes a variable name and returns a string or\n`nil`.","ref":"Nvir.Parser.html#interpolate_var/2"},{"type":"function","title":"Example - Nvir.Parser.interpolate_var/2","doc":"iex> envfile = \"\"\"\n    iex> GREETING=Hello $NAME!\n    iex> \"\"\"\n    iex> {:ok, [{\"GREETING\", variable}]} = Nvir.Parser.RDB.parse_string(envfile)\n    iex> resolver = fn \"NAME\" -> \"World\" end\n    iex> Nvir.Parser.interpolate_var(variable, resolver)\n    \"Hello World!\"","ref":"Nvir.Parser.html#interpolate_var/2-example"},{"type":"type","title":"Nvir.Parser.key/0","doc":"","ref":"Nvir.Parser.html#t:key/0"},{"type":"callback","title":"Nvir.Parser.parse_file/1","doc":"This callback must return a `{:ok, variables}` or `{:error, reason}` tuple.\n\nThe `variables` is a list of `{key, value}` tuples where:\n\n* `key` is a string\n* `value` is either a string or a `template`\n* A `template` is a list of `chunks`\n* A `chunk` is either a string or a `{:var, varname}` tuple.\n\nTemplates are used for interpolation. When a variable uses interpolation, the\nparser must not attempt to read the interpolated variables from environment,\nbut rather return a template instead of a binary value.\n\nVariables used in interpolation within other variables values do not have to\nexist in the file. Nvir uses a resolver to provide those values to execute the\ntemplates.\n\nIn this example, the `INTRO` variable is defined in the same file, but the\n`WHO` variable is not. This makes not difference, as the parser must not\nrequire those values.\n\n    iex> file_contents = \"INTRO=hello\\nGREETING=$INTRO $WHO!\"\n    iex> Nvir.Parser.RDB.parse_string(file_contents)\n    {:ok, [{\"INTRO\", \"hello\"}, {\"GREETING\", [{:var, \"INTRO\"}, \" \", {:var, \"WHO\"}, \"!\"]}]}\n\nYou can test and debug your parser by using `Nvir.Parser.interpolate_var/2`\nand a simple resolver.\n\n    iex> file_contents = \"GREETING=$INTRO $WHO!\"\n    iex> {:ok, [{\"GREETING\", template}]} = Nvir.Parser.RDB.parse_string(file_contents)\n    iex> resolver = fn\n    ...>   \"INTRO\" -> \"Hello\"\n    ...>   \"WHO\" -> \"World\"\n    ...> end\n    iex> Nvir.Parser.interpolate_var(template, resolver)\n    \"Hello World!\"","ref":"Nvir.Parser.html#c:parse_file/1"},{"type":"callback","title":"Expected results examples - Nvir.Parser.parse_file/1","doc":"Given this file content:\n\n```bash\nWHO=World\nGREETING=Hello $WHO!\n```\n\nThe `c:parse_file/1` callback should return the following:\n\n```elixir\n{:ok,\n  [\n    {\"WHO\", \"World\"},\n    {\"GREETING\", [\"Hello \", {:var, \"WHO\"}, \"!\"]}\n  ]}\n```\n\nWith this one using accumulative interpolation:\n\n```bash\nPATH=/usr/local/bin\nPATH=$PATH:/usr/bin\nPATH=/home/me/bin:$PATH\n```\n\nThe parser should produce the following:\n\n```elixir\n{:ok,\n  [\n    {\"PATH\", \"/usr/local/bin\"},\n    {\"PATH\", [{:var, \"PATH\"}, \":/usr/bin\"]},\n    {\"PATH\", [\"/home/me/bin:\", {:var, \"PATH\"}]}\n  ]}\n```","ref":"Nvir.Parser.html#c:parse_file/1-expected-results-examples"},{"type":"type","title":"Nvir.Parser.template/0","doc":"","ref":"Nvir.Parser.html#t:template/0"},{"type":"type","title":"Nvir.Parser.template_resolver/0","doc":"","ref":"Nvir.Parser.html#t:template_resolver/0"},{"type":"type","title":"Nvir.Parser.variable/0","doc":"","ref":"Nvir.Parser.html#t:variable/0"},{"type":"module","title":"Nvir.Parser.RDB","doc":"This is the default Nvir parser, a recursive-descent parser with backtracing\nsupport for dotenv files.","ref":"Nvir.Parser.RDB.html"},{"type":"type","title":"Nvir.Parser.RDB.buffer/0","doc":"","ref":"Nvir.Parser.RDB.html#t:buffer/0"},{"type":"macro","title":"Nvir.Parser.RDB.debug/2","doc":"","ref":"Nvir.Parser.RDB.html#debug/2"},{"type":"function","title":"Nvir.Parser.RDB.parse_file/1","doc":"Parses the given dotenv file.","ref":"Nvir.Parser.RDB.html#parse_file/1"},{"type":"function","title":"Nvir.Parser.RDB.parse_string/1","doc":"Parses the content of a dotenv file.","ref":"Nvir.Parser.RDB.html#parse_string/1"},{"type":"type","title":"Nvir.Parser.RDB.parser/0","doc":"","ref":"Nvir.Parser.RDB.html#t:parser/0"},{"type":"exception","title":"Nvir.CastError","doc":"Exception defining an invalid cast. For instance with this dotenv file:\n\n    NOT_AN_INT=some string\n\nCalling `Nvir.env!/2` with the `:integer!` caster will raise an\n`Nvir.CastError` exception.\n\n    Nvir.env!(\"NOT_AN_INT\", :integer!)","ref":"Nvir.CastError.html"},{"type":"exception","title":"Nvir.LoadError","doc":"Exception raised when `Nvir.dotenv!/1` fails to parse a dotenv file.","ref":"Nvir.LoadError.html"},{"type":"exception","title":"Nvir.Parser.ParseError","doc":"Exception representing parse errors from the default parser `Nvir.Parser`.","ref":"Nvir.Parser.ParseError.html"},{"type":"extras","title":"Nvir – Elixir environment variables made simple","doc":"# Nvir – Elixir environment variables made simple\n\n<!-- rdmx :badges\n    hexpm         : \"nvir?color=4e2a8e\"\n    github_action : \"lud/nvir/elixir.yaml?label=CI&branch=main\"\n    license       : nvir\n    -->\n[![hex.pm Version](https://img.shields.io/hexpm/v/nvir?color=4e2a8e)](https://hex.pm/packages/nvir)\n[![Build Status](https://img.shields.io/github/actions/workflow/status/lud/nvir/elixir.yaml?label=CI&branch=main)](https://github.com/lud/nvir/actions/workflows/elixir.yaml?query=branch%3Amain)\n[![License](https://img.shields.io/hexpm/l/nvir.svg)](https://hex.pm/packages/nvir)\n<!-- rdmx /:badges -->\nNvir is a powerful environment variable loader for Elixir that provides:\n\n* Simple loading of dotenv files with support for inheritance and interpolation.\n* Optional loading of dotenv files depending on the `:test`, `:dev` environment, CI environment, operating system, and custom conditions.\n* Strong validation and type casting of environment variables.\n* Support for custom casters and variables transformers.\n\nThis library is inspired from\n[Dotenvy](https://github.com/fireproofsocks/dotenvy) and provides a similar\nexperience.","ref":"readme.html"},{"type":"extras","title":"Installation - Nvir – Elixir environment variables made simple","doc":"As usual, pull the library from your `mix.exs` file.\n\n<!-- rdmx :app_dep runtime:false vsn:$app_vsn -->\n```elixir\ndef deps do\n  [\n    {:nvir, \"~> 0.13\", runtime: false},\n  ]\nend\n```\n<!-- rdmx /:app_dep -->","ref":"readme.html#installation"},{"type":"extras","title":"Basic Usage - Nvir – Elixir environment variables made simple","doc":"You will generally use Nvir from your `config/runtime.exs` file.\n\n* Import the module functions, and call `dotenv!/1` to load your files.\n* Use `env!/2` to require a variable and validate it.\n* Use `env!/3` to provide a default value.\n\n```elixir\n# runtime.exs\n\nimport Nvir\n\ndotenv!([\".env\", \".env.#{config_env()}\"])\n\nconfig :my_app, MyAppWeb.Endpoint,\n  secret_key_base: env!(\"SECRET_KEY_BASE\", :string!),\n  url: [host: env!(\"HOST\", :string!), port: 443, scheme: \"https\"],\n  http: [ip: {0, 0, 0, 0}, port: env!(\"PORT\", :integer!, 4000)]\n\nconfig :my_app, MyApp.Repo,\n  username: env!(\"DB_USERNAME\", :string!),\n  password: env!(\"DB_PASSWORD\", :string!),\n  database: env!(\"DB_DATABASE\", :string!),\n  hostname: env!(\"DB_HOSTNAME\", :string!),\n  port: env!(\"DB_PORT\", :integer!, 5432)\n```\n\nThis is most of what you need to know to start using this library.\n\nPlease refer to the documentation for advanced usage.","ref":"readme.html#basic-usage"},{"type":"extras","title":"Documentation - Nvir – Elixir environment variables made simple","doc":"Nvir provides advanced capabilities to work with your dotenv files in different\nscenarios. This is all described in the [documentation on\nhexdocs.pm](https://hexdocs.pm/nvir/readme.html) including the starter guides:\n\n* [Loading dotenv files](https://hexdocs.pm/nvir/loading-files.html)\n* [Reading environment variables](https://hexdocs.pm/nvir/the-env-functions.html)","ref":"readme.html#documentation"},{"type":"extras","title":"Interoperability - Nvir – Elixir environment variables made simple","doc":"This library is built around the native Elixir support for environment variables:\n\n* The `dotenv!/1` function will patch the actual runtime environment. You do not\n  have to use `env!/2` or `env!/3` to fetch variables loaded by `dotenv!/1`.\n  Using `System.fetch_env!/1`, `System.get_env/2`, etc. is perfectly fine.\n* The `env!` functions are helpers built around `System.fetch_env!/1` with\n  support for casting. They do not require `dotenv!/1` to have been called\n  beforehand and are safe to call wherever you would call `System.fetch_env!/1`\n  instead.\n\nYou may also use another library like\n[Enviable](https://github.com/halostatue/enviable) that provides more advanced\ncasters:\n\n```elixir\nimport Nvir\nimport Enviable\n\ndotenv!(\".env\")\n\nsecret_key = fetch_env_as_pem!(\"SECRET_KEY\")\ndns_config = fetch_env_as_json!(\"DNS_CONFIG_JSON\")\n```","ref":"readme.html#interoperability"},{"type":"extras","title":"Difference with Dotenvy - Nvir – Elixir environment variables made simple","doc":"Dotenvy considers the dotenv files to be configuration helpers only. They are\nonly available in the Elixir process executing `runtime.exs`.\n\nThe `System.fetch_env/1` function and other variants cannot access those\nvariables. It is possible to make Dotenvy actually declare system variables with\na _side effect_, but then their `env!` function will not find the variables.\n\nNvir philosophy is that the dotenv files are patches for the environment\n_around_ the application, and so the application should be able to use those\nvariables from anywhere with `System.fetch_env/1` or `Nvir.env!/2`.\n\nNvir will _always_ patch the system environment. The development environment and\nproduction environment will use the same code paths. This also works well with\nlibraries that expect some environment variables to be defined.","ref":"readme.html#difference-with-dotenvy"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\n## [0.13.0] - 2025-03-28","ref":"changelog.html"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- Only define functions with 'env' in their name for 'import Nvir'","ref":"changelog.html#features"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Update Elixir workflow (#18)\n\n## [0.12.0] - 2025-03-19","ref":"changelog.html#miscellaneous-tasks"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- [**breaking**] Removed the :* tag as it was confusing","ref":"changelog.html#features"},{"type":"extras","title":"📚 Documentation - Changelog","doc":"- Document dotenv!/env! independence (#16)\n\n## [0.11.0] - 2025-03-14","ref":"changelog.html#documentation"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- Added a hook to change variables before they are set","ref":"changelog.html#features"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Fix exemples with custom MIX_ENV\n- Remove unknown function in docs\n- Update dependabot config (#10)\n- Update Elixir workflow (#15)\n- Refactor collecting sources\n\n## [0.10.2] - 2025-01-31","ref":"changelog.html#miscellaneous-tasks"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Fixed parsed when ending with comment without final newline\n- Fixed parsed when ending with comment without final newline","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Update Elixir workflow (#8)\n- Update Elixir workflow (#9)\n\n## [0.10.1] - 2025-01-17","ref":"changelog.html#miscellaneous-tasks"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Trim trailing whitespace on unquoted strings\n\n## [0.10.0] - 2025-01-16","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- [**breaking**] Replace override with overwrite and support custom loaders","ref":"changelog.html#features"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Update dependabot config (#2)\n- Basic CI\n- Dummy CI change\n\n## [0.9.4] - 2024-11-25","ref":"changelog.html#miscellaneous-tasks"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"- Support old Elixir versions\n\n## [0.9.3] - 2024-11-18","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"🚀 Features - Changelog","doc":"- Default caster is :string\n\n## [0.9.2] - 2024-11-17","ref":"changelog.html#features"},{"type":"extras","title":"📚 Documentation - Changelog","doc":"- Document parsed templates\n\n## [0.9.1] - 2024-11-17","ref":"changelog.html#documentation"},{"type":"extras","title":"📚 Documentation - Changelog","doc":"- Added documentation to all functions","ref":"changelog.html#documentation"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Added changelog\n- README ordering\n- Versionning with mix version\n\n## [0.9.0] - 2024-11-17","ref":"changelog.html#miscellaneous-tasks"},{"type":"extras","title":"⚙️ Miscellaneous Tasks - Changelog","doc":"- Initialization of the repository\n- Added license\n\n<!-- generated by git-cliff -->","ref":"changelog.html#miscellaneous-tasks"},{"type":"extras","title":"Loading dotenv files","doc":"# Loading dotenv files\n\nThis page describes different scenarios for using the `dotenv!/1` function.\n\nDotenv files should always be loaded from your `config/runtime.exs` file.\nYou may have to create it yourself if it does not exist.","ref":"loading-files.html"},{"type":"extras","title":"Loading a single file - Loading dotenv files","doc":"This is the classic dotenv experience.\n\n```elixir\n# config/runtime.exs\n\n# Import the Config and Nvir modules\nimport Config\nimport Nvir\n\n# Load your dotenv file\ndotenv!(\".env\")\n\n# Start configuring your application\nconfig :my_app, MyApp.Repo,\n  username: env!(\"DB_USERNAME\", :string!),\n  password: env!(\"DB_PASSWORD\", :string!),\n  database: env!(\"DB_DATABASE\", :string!),\n  hostname: env!(\"DB_HOSTNAME\", :string!),\n  port: env!(\"DB_PORT\", :integer!, 5432)\n```","ref":"loading-files.html#loading-a-single-file"},{"type":"extras","title":"Loading from different sources - Loading dotenv files","doc":"The `Nvir.dotenv!/1` function accepts different types of sources to define which\ndotenv files to load.\n\nThe sources can be different types of values like lists, nested tuples, _etc._,\nbut all of them must finally contain a file path..\n\nNvir accepts relative paths or absolute paths. Relative paths are relative to\n`File.cwd!()`, which is the directory containing `mix.exs`.\n\nSee the custom loaders documentation to change the relative path target.","ref":"loading-files.html#loading-from-different-sources"},{"type":"extras","title":"Loading multiple files - Loading dotenv files","doc":"You can pass a list of different files to the `dotenv!/1` function.\n\nNvir ignores the files that do not exist: your `.env` file will likely not be\npresent in production, and you may have a `.env.test` file but no `.env.dev`\nfile\n\n```elixir\ndotenv!([\".env\", \".env.#{config_env()}\", \".env.local\"])\n```","ref":"loading-files.html#loading-multiple-files"},{"type":"extras","title":"Tagged sources - Loading dotenv files","doc":"Nvir has a concept of _enabled_ or _disabled_ sources. This works by wrapping the dotenv paths in tagged tuples.\n\nThis gives you more control over the files that are loaded, and ensures that no\nfile will be loaded in production if the dotenv files are committed to Git by\nmistake and/or included in your releases.\n\nIn this example, a different file is loaded depending on the current Mix\nenvironment.\n\n```elixir\ndotenv!(\n  dev: \".env\",\n  test: \".env.test\"\n)\n```\n\nIt is also valid to pass the same key multiple times:\n\n```elixir\ndotenv!(\n  dev: \".env\",\n  test: \".env.test\"\n  test: \".env.test.local\"\n)\n```","ref":"loading-files.html#tagged-sources"},{"type":"extras","title":"Predefined tags - Loading dotenv files","doc":"Those tags are defined automatically by Nvir based on the current environment.\n\nSee the custom loaders documentation to know how to define your own tags.\n\n#### Mix environment\n\n* `:dev` - When `Config.config_env()` or `Mix.env()` is `:dev`.\n* `:test` - When `Config.config_env()` or `Mix.env()` is `:test`.\n\nThere is no predefined tag for `:prod`. Using dotenv files in production is an\nanti-pattern. The guide on custom loaders will help you if you really need to.\n\n#### Continuous integration\n\n* `:ci` - When the `CI` environment variable is `\"true\"`. This variable is\n  defined by most CI services.\n* `:ci@github` - When the `GITHUB_ACTIONS` environment variable is `\"true\"`.\n* `:ci@travis` - When the `TRAVIS` environment variable is `\"true\"`.\n* `:ci@circle` - When the `CIRCLECI` environment variable is `\"true\"`.\n* `:ci@gitlab` - When the `GITLAB_CI` environment variable is `\"true\"`.\n\n#### Operating system\n\n* `:linux` - On Linux machines.\n* `:windows` - On Windows machines.\n* `:darwin` - On MacOS machines.","ref":"loading-files.html#predefined-tags"},{"type":"extras","title":"Nested sources - Loading dotenv files","doc":"List and tuple source may contain other nested _sources_, they are not limited to paths.\n\n\n```elixir\ndotenv!(\n  dev: \".env\",\n  test: [\".env.test\", \".env.test.local\", ci: \".env.ci\"]\n)\n```\n\nIn this example, the `:test` tag contains another list, and one element of this\nlist is a `:ci` tagged tuple.\n\nIf you are not familiar with Elixir's keyword lists, the following is an equivalent without the syntactic sugar.\n\n```elixir\ndotenv!([\n  {:dev, \".env\"},\n  {:test, [\".env\", \".env.test\", {:ci, \".env.ci\"}]}\n])\n```","ref":"loading-files.html#nested-sources"},{"type":"extras","title":"Overwrite mechanics - Loading dotenv files","doc":"The files loaded by Nvir will not replace variables already defined in the real\nenvironment.\n\nThat is, as your `HOME` variable already exists, defining `HOME=/somewhere/else`\nin a dotenv file will have no effect.\n\nA special tag can be given to `dotenv!/1` to overwrite system variables:\n\n```elixir\ndotenv!([\".env\", overwrite: \".env.local\"])\n```\n\nWith the code above, any variable from `.env` that does not already exist will\nbe added to the system env, but _all_ variables from `.env.local` will be set.\n\nJust like any source tag, the `:overwrite` key accepts any nested source types.\nThe following forms are equivalent:\n\n```elixir\ndotenv!(\n  dev: [\".env.dev\", overwrite: \".env.local.dev\"],\n  test: [\".env.test\", overwrite: \".env.local.test\"]\n)\n\ndotenv!(\n  dev: \".env.dev\",\n  test: \".env.test\",\n  overwrite: [dev: \".env.local.dev\", test: \".env.local.test\"]\n)\n```\n\nIn `:dev` environment, the two snippets above would both result in loading\n`.env.dev` then `.env.local.dev`.\n\nNesting `:overwrite` tags has no effect. All sources nested in an `:overwrite`\ntag are considered overwrites. In the following snippet, all files except\n`1.env` are overwrite files.\n\n```elixir\ndotenv!(\n  dev: \"1.env\",\n  overwrite: [\"2.env\", dev: [\"3.env\", overwrite: \"4.env\"]]\n)\n```\n\nThe `3.env` file is considered wrapped in an `:overwrite` tag, albeit\nindirectly. The `:overwrite` tag around `4.env` is useless.","ref":"loading-files.html#overwrite-mechanics"},{"type":"extras","title":"Load order - Loading dotenv files","doc":"The `dotenv!/1` function follows a couple rules when loading multiple sources:\n\n* Files are separated in two groups, \"regular\" and \"overwrites\".\n* Within each group, files are always loaded in order of appearance in the\n  sources list. This is important for files that reuse variables defined in\n  previous files.\n* The \"regular\" group is loaded first. The files from the \"overwrite\" group will\n  see the variables defined by the \"regular\" group.\n\nThe order of execution is the following:\n\n* Load all regular files in order.\n* Patch system environment with non-existing keys.\n* Load all overwrite files in order.\n* Overwrite system environment with all keys.\n\nThis means that the following expression will **not** load and apply\n`.env.local` first because it belongs to the \"overwrite\" group, which is applied\nlast. But `.env1` will always be loaded before `.env2`.\n\n```elixir\ndotenv!(overwrite: \".env.local\", dev: \".env1\", dev: \".env2\")\n```\n\nIn the following example, files are named in load order (without regard for\nenabled or disabled tags).\n\n```elixir\ndotenv!(\n  dev: \"1\",\n  test: [\"2\", ci: \"3\", overwrite: \"100\"]\n  overwrite: [\"101\", test: \"102\"]\n  linux: \"4\",\n)\n```\n\n**The load order impacts variable interpolation and inheritance for variables\nthat are repeated in multiple files.** Please refer to the Variable Inheritance\nguide.","ref":"loading-files.html#load-order"},{"type":"extras","title":"Custom loaders","doc":"# Custom loaders\n\nNvir allows you to customize the way the files are loaded. The order of the\nfiles is deterministic and cannot be changed, but options exist to change how\nthey are loaded.","ref":"custom-loaders.html"},{"type":"extras","title":"Using a custom loader - Custom loaders","doc":"The simple way is to start from the default loader and change its properties.\n\n\n```elixir\n# runtime.exs\nimport Config\nimport Nvir\n\ndotenv_loader()\n|> enable_sources(:docs, config_env() == :docs)\n|> enable_sources(:release, env!(\"RELEASE_NAME\", :boolean, false))\n|> dotenv_configure(cd: \"/app/release/env\")\n|> dotenv!(\n  dev: \".env\",\n  test: \".env.test\",\n  docs: \".env.docs\",\n  release: \"/var/release.env\"\n)\n```\n\nIn the example above, we will enable the `:docs` and `:release` tags when the\ndefined conditions are met.\n\nPlus, we changed the directory where the dotenv files are loaded from. This will\nnot affect the `/var/release.env` file since it's an absolute path.\n\nPlease refer to the documentation of `Nvir.dotenv_configure/2` to learn more\nabout the available options.","ref":"custom-loaders.html#using-a-custom-loader"},{"type":"extras","title":"Disabling default tags - Custom loaders","doc":"It is also possible to redefine predefined tags. Here we replace the `:test` tag\nwith a possibly different boolean value.\n\n```elixir\n# runtime.exs\nimport Config\nimport Nvir\n\ndotenv_loader()\n|> enable_sources(:test, config_env() == :test and MyApp.some_custom_check())\n|> dotenv!(\n  dev: \".env\",\n  test: \".env.test\"\n)\n```\n\nThe `:overwrite` tag cannot be changed, as it is handled separately from other\ntags.","ref":"custom-loaders.html#disabling-default-tags"},{"type":"extras","title":"Disabling all tags by default - Custom loaders","doc":"Use `dotenv_new()` instead of `dotenv_loader()` to get an empty loader without\nany enabled tag.","ref":"custom-loaders.html#disabling-all-tags-by-default"},{"type":"extras","title":"Using a custom parser - Custom loaders","doc":"If you want to parse the dotenv files yourself, or add support for other file\nformats, pass an implementation of the `Nvir.Parser` behaviour as the `:parser`\noption:\n\n```elixir\n# runtime.exs\nimport Config\nimport Nvir\n\ndotenv_new()\n|> dotenv_configure(parser: MyApp.YamlEnvParser)\n|> dotenv!(\"priv/dev-env.yaml\")\n```","ref":"custom-loaders.html#using-a-custom-parser"},{"type":"extras","title":"Transforming the variables - Custom loaders","doc":"It is possible to change the keys and values of the variables before they are\ndefined in the environment, by using the `:before_env_set` hook.\n\nThe function is passed a tuple with the variable name and value, and must return\na name and value.\n\nThe returned name and value must be encodable as strings using the `to_string/1`\nElixir function.\n\nIt _is_ possible to return a different name from there. The original variable name will _not_ be defined. We use this in the example below but it's generally not recommended for clarity's sake.\n\nExample:\n\n```elixir\n# runtime.exs\nimport Config\nimport Nvir\n\nto_homepage = fn username ->\n  uri = URI.parse(\"http://example.com/\")\n  %{uri | path: \"/\" <> username}\nend\n\ndotenv_new()\n|> dotenv_configure(\n  before_env_set: fn\n    {\"USERNAME\", username} -> {:HOMEPAGE, to_homepage.(username)}\n    other -> other\n  end\n)\n|> dotenv!(\".env\")\n```\n\n* The transformation returns a different variable name.\n* The `USERNAME` variable will not be set by Nvir.\n* The `HOMEPAGE` variable is returned with an atom key and a `URI` struct value.\n* Nvir will set both key and values as strings in the System environment.","ref":"custom-loaders.html#transforming-the-variables"},{"type":"extras","title":"Reading Environment Variables","doc":"# Reading Environment Variables\n\n\nThe `Nvir.env!/1`, `Nvir.env!/2` and `Nvir.env!/3` functions allow you to load\nan environment variable and cast it to the appropriate type by passing an\noptional caster:\n\n```elixir\nimport Nvir\n\n# This will raise if the variable is not defined\nhost = env!(\"HOST\")\n\n# This will raise if the variable is not defined or is empty\nhost = env!(\"HOST\", :string!)\n\n# This will use a default value if the variable is not defined, or otherwise\n# convert the value to an integer.\nport = env!(\"PORT\", :integer!, 4000)\n```","ref":"the-env-functions.html"},{"type":"extras","title":"Requiring a variable - Reading Environment Variables","doc":"To fetch a variable, call `env!/2` with the variable name and an optional\ncaster. A caster is either a built-in caster (given as an atom) or a custom\ncaster. See later for a description of both.\n\n```elixir\nenv!(\"PORT\", :integer!)\n```\n\nThis will attempt to fetch the variable as in `System.fetch_env!(\"SOME_KEY\")`, cast its value to an integer and return that value.\n\nTwo exceptions may be raised from that call:\n\n* `System.EnvError` if the variable is not defined.\n* `Nvir.CastError` if the cast fails.\n\nIf no caster is given, Nvir will use the `:string` caster:\n\n```elixir\nenv!(\"HOST\")\n```\n\nEnvironment variables are always strings, so the `:string` caster will return\nthe value as-is, given it is defined.","ref":"the-env-functions.html#requiring-a-variable"},{"type":"extras","title":"Default values - Reading Environment Variables","doc":"Use `env!/3` to provide a default value.\n\nDefault values are not used when a variable is found, even if the cast fails, if the variable is empty, or whatever.\n\n```elixir\nenv!(\"PORT\", :integer!, 4000)\n```\n\nDefault values are not validated by the caster:\n\n```elixir\n# :infinity is not a valid integer but this works\nenv!(\"TIMEOUT\", :integer!, :infinity)\n```","ref":"the-env-functions.html#default-values"},{"type":"extras","title":"Built-in Casters - Reading Environment Variables","doc":"Built-in casters are defined as atoms. There are three flavors that behave\ndifferently when an environment variable value is an empty string.\n\n* Casters suffixed with `!` like `:integer!` or `:string!` will raise if the\n  variable contains an empty string.\n* Casters suffixed with `?` like `:integer?` or `:string?` will convert empty\n  strings to `nil` instead of casting.\n* Casters without a suffix exist for types that can be cast from an empty\n  string, _i.e._ `:string`, `:atom`, `:existing_atom` and `:boolean`.\n\nSee below for a complete list of built-in casters and custom casters.\n\nEmpty strings occur when a variable is defined without a value:\n\n```\nHOST=localhost # value is \"localhost\"\nPORT=          # value is \"\"\n```\n\nRemember, as long as the key exists, the default value is never used; this holds\ntrue for empty string values.\n\nWith `PORT=\"\"` in the environment:\n\n* Calling `env!(\"PORT\", :integer!, 4000)` will raise because `\"\"` can't be cast\n  to an integer.\n* Calling `env!(\"PORT\", :integer?, 4000)` will return `nil`.\n\n<!-- rdmx :section name:available_casters -->","ref":"the-env-functions.html#built-in-casters"},{"type":"extras","title":"String Casters - Reading Environment Variables","doc":"| Caster | Description |\n|--------|-------------|\n| `:string` | Returns the value as-is. |\n| `:string?` | Converts empty strings to `nil`. |\n| `:string!` | Raises for empty strings. |","ref":"the-env-functions.html#string-casters"},{"type":"extras","title":"Boolean Casters - Reading Environment Variables","doc":"| Caster | Description |\n|--------|-------------|\n| `:boolean` | `\"false\"`, `\"0\"` and empty strings become `false`, any other value is `true`. Case insensitive. It is recommended to use `:boolean!` instead. |\n| `:boolean!` | Accepts only `\"true\"`, `\"false\"`, `\"1\"`, and `\"0\"`. Case insensitive. |","ref":"the-env-functions.html#boolean-casters"},{"type":"extras","title":"Number Casters - Reading Environment Variables","doc":"| Caster | Description |\n|--------|-------------|\n| `:integer!` | Strict integer parsing. |\n| `:integer?` | Like `:integer!` but empty strings becomes `nil`. |\n| `:float!` | Strict float parsing. |\n| `:float?` | Like `:float!` but empty strings becomes `nil`. |","ref":"the-env-functions.html#number-casters"},{"type":"extras","title":"Atom Casters - Reading Environment Variables","doc":"| Caster | Description |\n|--------|-------------|\n| `:atom` | Converts the value to an atom. Use the `:existing_atom` variants when possible. |\n| `:atom?` | Like `:atom` but empty strings becomes `nil`. |\n| `:atom!` | Like `:atom` but rejects empty strings. |\n| `:existing_atom` | Converts to existing atom only, raises otherwise. |\n| `:existing_atom?` | Like `:existing_atom` but empty strings becomes `nil`. |\n| `:existing_atom!` | Like `:existing_atom` but rejects empty strings. |\n\nNote that using `:existing_atom` with empty strings will not raise an exception\nbecause the `:\"\"` atom is valid and is generally defined by the system on boot.","ref":"the-env-functions.html#atom-casters"},{"type":"extras","title":"Deprecated casters - Reading Environment Variables","doc":"Those exist for legacy reasons and should be avoided. They will trigger a\nruntime warning when used.\n\nIn some languages, using `null` where a number is expected will cast the value\nto a _default type value_, generally `0` and `+0.0` for integers and floats.\nThis behaviour does not exist in Elixir so casters for such types behave the\nsame with-or-without the `!` suffix. This means `:integer` and `:float` will\nraise for empty strings.\n\n| Caster | Description |\n|--------|-------------|\n| `:boolean?` | Same as `:boolean`. ⚠️ Returns `false` instead of `nil` for empty strings. |\n| `:integer` | Same as `:integer!`. |\n| `:float` | Same as `:float!`. |\n\n<!-- rdmx /:section -->","ref":"the-env-functions.html#deprecated-casters"},{"type":"extras","title":"Custom Casters - Reading Environment Variables","doc":"The second argument to `env!/2` and `env/3` also accept custom validators using\nan anonymous function. The given function must return `{:ok, value}` or\n`{:error, message}` where `message` is a string.\n\n```elixir\nenv!(\"URL\", fn\n  \"https://\" <> _ = url -> {:ok, url}\n  _ -> {:error, \"https is required\"}\nend)\n```\n\nIt is also possible to return errors from `Nvir.Cast.cast/2`. (Those are not\nstrings but they are properly handled.)\n\n\n```elixir\nenv!(\"PORT\", fn value ->\n  case Nvir.Cast.cast(value, :integer!) do\n    {:ok, port} when port > 1024 -> {:ok, port}\n    {:ok, port} -> {:error, \"invalid port: #{port}\"}\n    {:error, reason} -> {:error, reason}\n  end\nend)\n```","ref":"the-env-functions.html#custom-casters"},{"type":"extras","title":"Dotenv File Syntax Reference","doc":"# Dotenv File Syntax Reference","ref":"dotenv-syntax.html"},{"type":"extras","title":"Basic Syntax - Dotenv File Syntax Reference","doc":"```elixir\n# Simple assignment\nKEY=value\n\n# With spaces around =\nKEY = value\n\n# Empty value\nEMPTY=\nEMPTY=\"\"\nEMPTY=''\n\n# The parser will ignore an \"export\" prefix\nexport KEY=value\n\n# Interpolation with previously defined variable\nPATH=/usr/bin\nPATH=$PATH:/home/alice/bin\nPATH=/usr/local/bin:$PATH\n```","ref":"dotenv-syntax.html#basic-syntax"},{"type":"extras","title":"Comments - Dotenv File Syntax Reference","doc":"Comments are supported on their own line or at the end of a line.\n\n**Important**, when a value is not quoted, the comment `#` character must be\nseparated by at least one space, otherwise the comment will be included in the\nvalue.\n\n```elixir\n# This is a comment on it's own line\n\nKEY=value # Inline comment\n\nKEY=value# No preceding space, this is part of the value\n```","ref":"dotenv-syntax.html#comments"},{"type":"extras","title":"Single-Line Values - Dotenv File Syntax Reference","doc":"","ref":"dotenv-syntax.html#single-line-values"},{"type":"extras","title":"Raw Strings - Dotenv File Syntax Reference","doc":"Quotes are optional around single-line values.\n\n```elixir\nKEY=raw value with spaces\n```","ref":"dotenv-syntax.html#raw-strings"},{"type":"extras","title":"Double Quotes - Dotenv File Syntax Reference","doc":"Double quotes let you write escape sequences and trailing whitespace.\n\n```elixir\nKEY=\"value with spaces\"\nKEY=\"escape \\\"quotes\\\" inside\"\nKEY=\"supports \\n \\r \\t \\b \\f escapes\"\nPREFIX=\"hello \"\n```","ref":"dotenv-syntax.html#double-quotes"},{"type":"extras","title":"Single Quotes - Dotenv File Syntax Reference","doc":"Single quotes define verbatim values. No escaping is done except for the single\nquote itself.\n\n```elixir\nKEY='value with spaces'\nKEY='no escapes \\n' # value will have a \"\\\" character followed by a \"n\".\nKEY='escape \\'quotes\\' inside'\n```","ref":"dotenv-syntax.html#single-quotes"},{"type":"extras","title":"Multiline Strings - Dotenv File Syntax Reference","doc":"The same rules apply for escaping as in single line values.","ref":"dotenv-syntax.html#multiline-strings"},{"type":"extras","title":"Triple Double Quotes - Dotenv File Syntax Reference","doc":"Double quotes let you write escape sequences\n\n```elixir\nKEY=\"\"\"\nLine 1\nLine 2 with \"quotes\"\n\"\"\"\n```","ref":"dotenv-syntax.html#triple-double-quotes"},{"type":"extras","title":"Triple Single Quotes - Dotenv File Syntax Reference","doc":"Single quotes define verbatim values. No escaping is done except for the single quote itself.\n\n```elixir\nKEY='''\nLine 1\nLine 2 with 'quotes'\n'''\n```","ref":"dotenv-syntax.html#triple-single-quotes"},{"type":"extras","title":"Trailing Whitespace - Dotenv File Syntax Reference","doc":"Trailing whitespace is automatically removed from the end of single-line values\nonly.\n\nIn this example, some spaces are represented with the `_` symbol to make it look\nmore explicit.\n\n```elixir\n# KEY will contain \"value\"\nKEY=value # Inline comment\n\n# KEY will contain \"value\" too\nKEY=value____\n\n# Multiline strings (with simple and double quotes) are NOT trimmed.\n# KEY will contain \"Hello!    \\nHow are you    \\n\"\nKEY=\"\"\"\nHello____\nHow are you____\n\"\"\"\n\n# Empty Whitespace is trimmed\n# KEY will contain \"\"\nKEY=____\n\n# Use quotes to express whitespace\nINDENT=\"    \"\n```","ref":"dotenv-syntax.html#trailing-whitespace"},{"type":"extras","title":"Variable Interpolation - Dotenv File Syntax Reference","doc":"Nvir supports variable interpolation within dotenv files.\n\nSingle quotes are not interpolated.\n\nPlease refer to the \"Variables Inheritance\" documentation for more details on\nwhich value is used on different setups.\n\n```elixir\n# This variable can be used below in the same file\nGREETING=Hello\n\n# Basic syntax\nMSG=$GREETING World\n\n# Enclosed syntax\nMSG=${GREETING} World\n\n# Not interpolated (single quotes)\nMSG='$GREETING World'\n\n# In raw values, a comment without a preceding space will be included in the\n# value\nMSG=$GREETING# This is part of the value\nMSG=${GREETING}# This too\nMSG=$GREETING # Actual comment\n```","ref":"dotenv-syntax.html#variable-interpolation"},{"type":"extras","title":"Variables Inheritance","doc":"# Variables Inheritance\n\nThis documentation describes how Nvir figures out which value to use when a\nvariable uses interpolation with other variables in a dotenv file.\n\n```\nGREETING=Hello $USERNAME!\n```","ref":"variables-inheritance.html"},{"type":"extras","title":"System inheritance - Variables Inheritance","doc":"When loading multiple files, Nvir will load the files in a deterministic order\nand splits those files into \"regular\" and \"overwrite\" files.\n\nSee the file loading documentation for more details. The important part here is\nthat it changes the inheritance behaviour.","ref":"variables-inheritance.html#system-inheritance"},{"type":"extras","title":"System inheritance for regular files - Variables Inheritance","doc":"Regular files do not overwrite the system environment, so the variables used in\ninterpolation always use the actual system environment variables first, and then\nfalls back to the variables defined in the dotenv files.\n\n\nIn this example with a regular file, the `WHO=moon` variable will not be set in\nthe environment as is it already defined in the system environment as\n`WHO=world`.\n\nNvir will respect that logic when building interpolated values:\n\n```elixir\n# System state:\n# WHO=world\n\n# .env, a regular file.\nWHO=moon\nGREETING=hello $WHO   # Will use WHO=world from system since we are not overwriting\n\n# Result:\n# GREETING=hello world\n```\n\nIf the variable does not already exist in the system, then it will use\n`WHO=moon` for interpolation because it will also define the variable in the\nruntime environment.","ref":"variables-inheritance.html#system-inheritance-for-regular-files"},{"type":"extras","title":"System inheritance for overwrite files - Variables Inheritance","doc":"When dealing with overwrite files, the logic is simpler, each variable is always\ndefined to the latest seen value, in interpolation as well as in the final set\nof variables added to the runtime environment.","ref":"variables-inheritance.html#system-inheritance-for-overwrite-files"},{"type":"extras","title":"Multiple files - Variables Inheritance","doc":"With multiple files, we use the latest value. In this exemple the variable is\nnot already defined in the system:\n\n```elixir\n# .env\nWHO=world\n\n# .env.dev\nWHO=mars\nGREETING=hello $WHO # This defines GREETING=hello mars\n\n# .env.dev.2\nWHO=moon\nGREETING=hello $WHO # this defines GREETING=hello moon\n\n# With loading order of .env, .env.dev, .env.dev.2\n# we will have the following:\n# WHO=moon\n# GREETING=hello moon\n```\n\nSo, \"regular\" files do not overwrite the system environment, but they act as a\ngroup and overwrite themselves as if they were a single file.\n\nIf the variable is already defined in the system, the values from all regular\nfiles are ignored just as in the single-file example.","ref":"variables-inheritance.html#multiple-files"},{"type":"extras","title":"Important edge case - Variables Inheritance","doc":"When a variable used in interpolation is later redefined in subsequent files,\ndependent variables that used its value will not use the last defined value:\n\n```elixir\n# .env\nWHO=world\n\n# .env.dev\nWHO=mars\nGREETING=hello $WHO     # GREETING is defined here, using WHO=mars\n\n# .env.dev.2\nWHO=moon                # WHO is updated, but GREETING keeps its value\n                        # since it's not redefined here\n\n# Final result:\n# WHO=moon\n# GREETING=hello mars  # The result is not \"hello moon\"\n```\n\nThis may cause inconsistencies if you code depends on the values of both\n`GREETING` and `WHO`.","ref":"variables-inheritance.html#important-edge-case"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}